use rocket::{
    http::CookieJar,
    serde::json::{json, Json},
    State,
};
use serenity::{client::Context, model::id::UserId};
use sqlx::{MySql, Pool};

use crate::{
    guards::transaction::Transaction,
    routes::{
        dashboard::api::user::models::{create_reminder, Reminder},
        JsonResult,
    },
};

#[post("/api/user/reminders", data = "<reminder>")]
pub async fn create_user_reminder(
    reminder: Json<Reminder>,
    cookies: &CookieJar<'_>,
    ctx: &State<Context>,
    mut transaction: Transaction<'_>,
) -> JsonResult {
    let user_id =
        cookies.get_private("userid").map(|c| c.value().parse::<u64>().ok()).flatten().unwrap();

    match create_reminder(
        ctx.inner(),
        &mut transaction,
        UserId::new(user_id),
        reminder.into_inner(),
    )
    .await
    {
        Ok(r) => match transaction.commit().await {
            Ok(_) => Ok(r),
            Err(e) => {
                warn!("Couldn't commit transaction: {:?}", e);
                json_err!("Couldn't commit transaction.")
            }
        },

        Err(e) => Err(e),
    }
}

#[get("/api/user/reminders")]
pub async fn get_reminders(
    cookies: &CookieJar<'_>,
    ctx: &State<Context>,
    pool: &State<Pool<MySql>>,
) -> JsonResult {
    let user_id =
        cookies.get_private("userid").map(|c| c.value().parse::<u64>().ok()).flatten().unwrap();
    let channel = UserId::new(user_id).create_dm_channel(ctx.inner()).await;

    match channel {
        Ok(channel) => sqlx::query_as_unchecked!(
            Reminder,
            "
            SELECT
                reminders.attachment,
                reminders.attachment_name,
                reminders.content,
                reminders.embed_author,
                reminders.embed_author_url,
                reminders.embed_color,
                reminders.embed_description,
                reminders.embed_footer,
                reminders.embed_footer_url,
                reminders.embed_image_url,
                reminders.embed_thumbnail_url,
                reminders.embed_title,
                IFNULL(reminders.embed_fields, '[]') AS embed_fields,
                reminders.enabled,
                reminders.expires,
                reminders.interval_seconds,
                reminders.interval_days,
                reminders.interval_months,
                reminders.name,
                reminders.tts,
                reminders.uid,
                reminders.utc_time
            FROM reminders
            INNER JOIN channels ON channels.id = reminders.channel_id
            WHERE `status` = 'pending' AND channels.channel = ?
            ",
            channel.id.get()
        )
        .fetch_all(pool.inner())
        .await
        .map(|r| Ok(json!(r)))
        .unwrap_or_else(|e| {
            warn!("Failed to complete SQL query: {:?}", e);

            json_err!("Could not load reminders")
        }),

        Err(e) => {
            warn!("Couldn't get DM channel: {:?}", e);

            json_err!("Could not find a DM channel")
        }
    }
}
